@@ -40,7 +40,7 @@ setValidity("DESeqDataSet", function(object) {
       any(duplicated(factor.nms))
     }))) {
       return("levels of factors in the design have non-unique level names after make.names() is applied.
-  best to only uobject letters and numbers for levels of factors in the design")
+  best to only use letters and numbers for levels of factors in the design")
     }
     # levels contain characters other than letters, numbers, and underscore
     if (any(sapply(designFactors,function(v) {
@@ -144,6 +144,10 @@ DESeqDataSet <- function(se, design, ignoreRank=FALSE) {
     message("renaming the first element in assays to 'counts'")
     assayNames(se)[1] <- "counts"
   }
+  # special tximeta processing
+  if ("tximetaInfo" %in% names(metadata(se))) {
+    se <- processTximeta(se)
+  }  
   # before validity check, try to convert assay to integer mode
   if (any(is.na(assay(se))))
     stop("NA values are not allowed in the count matrix")
@@ -185,8 +189,13 @@ DESeqDataSet <- function(se, design, ignoreRank=FALSE) {
     if (!all(designVars %in% names(colData(se)))) {
       stop("all variables in design formula must be columns in colData")
     }
+    for (v in designVars) {
+      if (any(is.na(colData(se)[[v]])))
+        stop(paste("variables in design formula cannot contain NA:", v))
+    }
+    
+    designVarsClass <- sapply(designVars, function(v) class(colData(se)[[v]])[1])
 
-    designVarsClass <- sapply(designVars, function(v) class(colData(se)[[v]]))
     if (any(designVarsClass == "character")) {
       warning("some variables in design formula are characters, converting to factors")
       for (v in designVars[designVarsClass == "character"]) {
@@ -211,13 +220,19 @@ DESeqDataSet <- function(se, design, ignoreRank=FALSE) {
         }
       }
       if (warnIntVars) {
-        message("the design formula contains a numeric variable with integer values,
+        message("  the design formula contains a numeric variable with integer values,
   specifying a model with increasing fold change for higher values.
   did you mean for this to be a factor? if so, first convert
   this variable to a factor using the factor() function")
       }
     }
 
+    if (any(designVarsClass == "ordered")) {
+      stop("the design formula contains an ordered factor. The internal steps
+do not work on ordered factors as a formula. Instead you should provide a matrix to
+the 'design' slot or to the 'full' argument of DESeq(), constructed using model.matrix.")
+    }
+    
     designFactors <- designVars[designVarsClass == "factor"]
     missingLevels <- sapply(designFactors, function(v) any(table(colData(se)[[v]]) == 0))
     if (any(missingLevels)) {
@@ -243,7 +258,7 @@ DESeqDataSet <- function(se, design, ignoreRank=FALSE) {
       for (cSyn in controlSynonyms) {
         if (cSyn %in% lastDVLvls) {
           if (cSyn != lastDVLvls[1]) {
-            message(paste0("it appears that the last variable in the design formula, '",designVars[lastDV],"',
+            message(paste0("  it appears that the last variable in the design formula, '",designVars[lastDV],"',
   has a factor level, '",cSyn,"', which is not the reference level. we recommend
   to use factor(...,levels=...) or relevel() to set this as the reference level
   before proceeding. for more information, please see the 'Note on factor levels'
@@ -347,10 +362,11 @@ DESeqDataSetFromMatrix <- function( countData, colData, design, tidy=FALSE, igno
 DESeqDataSetFromHTSeqCount <- function( sampleTable, directory=".", design, ignoreRank=FALSE, ...) 
 {
   if (missing(design)) stop("design is missing")
-  l <- lapply( as.character( sampleTable[,2] ), function(fn) read.table( file.path( directory, fn ) ) )
+  l <- lapply( as.character( sampleTable[,2] ), function(fn) read.table( file.path( directory, fn ), fill=TRUE ) )
   if( ! all( sapply( l, function(a) all( a$V1 == l[[1]]$V1 ) ) ) )
     stop( "Gene IDs (first column) differ between files." )
-  tbl <- sapply( l, function(a) a$V2 )
+  # select last column of 'a', works even if htseq was run with '--additional-attr'
+  tbl <- sapply( l, function(a) a[,ncol(a)] )
   colnames(tbl) <- sampleTable[,1]
   rownames(tbl) <- l[[1]]$V1
   rownames(sampleTable) <- sampleTable[,1]
@@ -445,3 +461,21 @@ DESeqTransform <- function(SummarizedExperiment) {
   }
   new("DESeqTransform", se)
 }
+
+# unexported helper function
+processTximeta <- function(se) {
+  assay(se) <- round(assay(se))
+  mode(assay(se)) <- "integer"
+  stopifnot("countsFromAbundance" %in% names(metadata(se)))
+  cfa <- metadata(se)$countsFromAbundance
+  stopifnot(cfa %in% c("no","scaledTPM","lengthScaledTPM"))  
+  if (cfa %in% c("scaledTPM","lengthScaledTPM")) {
+    message("using just counts from tximeta")
+  } else {
+    message("using counts and average transcript lengths from tximeta")
+    stopifnot(all(assays(se)$length > 0))
+    anms <- assayNames(se)
+    assayNames(se)[anms == "length"] <- "avgTxLength"
+  }
+  return(se)
+}
