@@ -434,6 +434,8 @@ of length 3 to 'contrast' instead of using 'name'")
     if (is.list(contrast)) stop("addMLE only implemented for: contrast=c('condition','B','A')")
     res <- cbind(res, mleContrast(object, contrast))
     res <- res[,c("baseMean","log2FoldChange","lfcMLE","lfcSE","stat","pvalue")]
+    # if an all zero contrast, also zero out the lfcMLE
+    res$lfcMLE[ which(res$log2FoldChange == 0 & res$stat == 0) ] <- 0
   }
   
   # only if we need to generate new p-values
@@ -482,25 +484,47 @@ of length 3 to 'contrast' instead of using 'name'")
   m <- nrow(attr(object,"dispModelMatrix"))
   p <- ncol(attr(object,"dispModelMatrix"))
   
-  # only if more samples than parameters:
-  if (m > p) {
-    defaultCutoff <- qf(.99, p, m - p)
-    if (missing(cooksCutoff)) {
-      cooksCutoff <- defaultCutoff
-    }
-    stopifnot(length(cooksCutoff)==1)
-    if (is.logical(cooksCutoff) & cooksCutoff) {
-      cooksCutoff <- defaultCutoff
-    }
-  } else {
-    cooksCutoff <- FALSE
+  defaultCutoff <- qf(.99, p, m - p)
+  if (missing(cooksCutoff)) {
+    cooksCutoff <- defaultCutoff
+  }
+  stopifnot(length(cooksCutoff)==1)
+  if (is.logical(cooksCutoff) & cooksCutoff) {
+    cooksCutoff <- defaultCutoff
   }
   
   # apply cutoff based on maximum Cook's distance
   performCooksCutoff <- (is.numeric(cooksCutoff) | cooksCutoff) 
-  if ((m > p) & performCooksCutoff) {
+  if (performCooksCutoff) {
     cooksOutlier <- mcols(object)$maxCooks > cooksCutoff
-    res$pvalue[cooksOutlier] <- NA
+    
+    ### BEGIN heuristic to avoid filtering genes with low count outliers
+    # as according to Cook's cutoff. only for two group designs.
+    # dont filter if three or more counts are larger
+    if (any(cooksOutlier,na.rm=TRUE) & is(design(object), "formula")) {
+      designVars <- all.vars(design(object))
+      if (length(designVars) == 1) {
+        var <- colData(object)[[designVars]]
+        if (is(var, "factor") && nlevels(var) == 2) {
+          dontFilter <- logical(sum(cooksOutlier,na.rm=TRUE))
+          for (i in seq_along(dontFilter)) {
+            # index along rows of object
+            ii <- which(cooksOutlier)[i] 
+            # count for the outlier with max cooks
+            outCount <- counts(object)[ii,which.max(assays(object)[["cooks"]][ii,])]
+            # if three or more counts larger than the outlier
+            if (sum(counts(object)[ii,] > outCount) >= 3) {
+              # don't filter out the p-value for that gene
+              dontFilter[i] <- TRUE
+            }
+          }
+          # reset the outlier status for these genes
+          cooksOutlier[which(cooksOutlier)][dontFilter] <- FALSE
+        }
+      }
+    } ### END heuristic ###
+
+    res$pvalue[which(cooksOutlier)] <- NA
   }
 
   # if original baseMean was positive, but now zero due to replaced counts, fill in results
@@ -547,26 +571,10 @@ of length 3 to 'contrast' instead of using 'name'")
     rownames(deseqRes) <- NULL
     deseqRes <- as.data.frame(deseqRes)
   }
-  
-  if (format == "DataFrame") {
-    return(deseqRes)
-  } else if (format == "GRangesList") {
-    if (is(rowRanges(object), "GRanges")) warning("rowRanges is GRanges")
-    out <- rowRanges(object)
-    mcols(out) <- deseqRes
-    return(out)
-  } else if (format == "GRanges") {
-    if (is(rowRanges(object), "GRangesList")) {
-      message("rowRanges is GRangesList, performing unlist(range(x)) on the rowRanges")
-      out <- unlist(range(rowRanges(object)))
-      mcols(out) <- deseqRes
-      return(out)
-    } else {
-      out <- rowRanges(object)
-      mcols(out) <- deseqRes
-      return(out)
-    }
-  }
+
+  # return DataFrame, GRanges or GRangesList
+  out <- resultsFormatSwitch(object=object, res=deseqRes, format=format)
+  return(out)
 }
 
 #' @rdname results
@@ -868,6 +876,11 @@ cleanContrast <- function(object, contrast, expanded=FALSE, listValues, test, us
                    "are expected to be in resultsNames(object)"))
       }
     }
+
+    # check if both levels have all zero counts
+    # (this has to be down here to make use of error checking above)
+    contrastAllZero <- contrastAllZeroCharacter(object, contrastFactor,
+                                                contrastNumLevel, contrastDenomLevel)
     
   }
 
@@ -912,6 +925,8 @@ cleanContrast <- function(object, contrast, expanded=FALSE, listValues, test, us
       contrastName <- paste(contrastFactor,contrastNumLevel,"vs",contrastDenomLevel)
     }
 
+    contrastAllZero <- contrastAllZeroNumeric(object, contrast)
+
     # now get the contrast
     contrastResults <- getContrast(object, contrast, useT=useT, minmu)
     lfcType <- if (attr(object,"betaPrior")) "MAP" else "MLE"
@@ -924,9 +939,18 @@ cleanContrast <- function(object, contrast, expanded=FALSE, listValues, test, us
                                         description=contrastDescriptions)
     res <- cbind(mcols(object)["baseMean"],
                  contrastResults)
-    
+
   }
-  
+    
+  # if the counts in all samples included in contrast are zero
+  # then zero out the LFC, Wald stat and p-value set to 1
+  contrastAllZero <- contrastAllZero & !mcols(object)$allZero
+  if (sum(contrastAllZero) > 0) {
+    res$log2FoldChange[contrastAllZero] <- 0
+    res$stat[contrastAllZero] <- 0
+    res$pvalue[contrastAllZero] <- 1
+  }
+
   # if test is "LRT", overwrite the statistic and p-value
   # (we only ran contrast for the coefficient)
   if (test == "LRT") {
@@ -1100,3 +1124,61 @@ or the denominator (second element of contrast list), but not both")
   return(contrast)
 }
 
+# function to determine output of results() and lfcShrink()
+resultsFormatSwitch <- function(object, res, format) {
+  if (format == "DataFrame") {
+    return(res)
+  } else if (format == "GRangesList") {
+    if (is(rowRanges(object), "GRanges")) warning("rowRanges is GRanges")
+    out <- rowRanges(object)
+    mcols(out) <- res
+    return(out)
+  } else if (format == "GRanges") {
+    if (is(rowRanges(object), "GRangesList")) {
+      message("rowRanges is GRangesList, performing unlist(range(x)) on the rowRanges")
+      out <- unlist(range(rowRanges(object)))
+      mcols(out) <- res
+      return(out)
+    } else {
+      out <- rowRanges(object)
+      mcols(out) <- res
+      return(out)
+    }
+  }
+}
+
+
+contrastAllZeroCharacter <- function(object, contrastFactor,
+                                     contrastNumLevel, contrastDenomLevel) {
+  cts <- counts(object)
+  f <- colData(object)[[contrastFactor]]
+  cts.sub <- cts[ , f %in% c(contrastNumLevel, contrastDenomLevel), drop=FALSE ]
+  rowSums( cts.sub == 0 ) == ncol(cts.sub)
+}
+
+contrastAllZeroNumeric <- function(object, contrast) {
+  if (is.null(attr(object,"modelMatrix"))) {
+    stop("was expecting a model matrix stored as an attribute of the DESeqDataSet")
+  }
+  modelMatrix <- attr(object, "modelMatrix")
+  # note: this extra leg-work to zero out LFC, lfcSE, and set p-value to 1
+  # for contrasts comparing groups where both groups have all zeros.
+
+  # it is only implemented for the case in which we can identify
+  # the relevant samples by multiplying the model matrix
+  # with a vector where the non-zero elements of the numeric contrast
+  # are replaced with 1.
+
+  # so, this code will not zero out in the case of standard model matrices
+  # where the user supplies a numeric contrast that pulls out a single column
+  # of the model matrix, for example.
+  
+  if (all(contrast >= 0) | all(contrast <= 0)) {
+    return( rep(FALSE, nrow(object)) )
+  }
+  
+  contrastBinary <- ifelse(contrast == 0, 0, 1)
+  whichSamples <- ifelse(modelMatrix %*% contrastBinary == 0, 0, 1)
+  zeroTest <- counts(object) %*% whichSamples
+  zeroTest == 0
+}
